@@ -194,7 +194,7 @@ void GfxFont::readFontDescriptor(XRef *xref, Dict *fontDict) {
       embFontID = obj2.getRef();
       if (type != fontType1) {
 	error(-1, "Mismatch between font type and embedded font file");
-	type = fontType1;
+	type = isCIDFont() ? fontCIDType0 : fontType1;
       }
     }
     obj2.free();
