@@ -703,7 +703,7 @@ void SplashOutputDev::startDoc(XRef *xrefA) {
   nT3Fonts = 0;
 }
 
-void SplashOutputDev::startPage(int pageNum, GfxState *state, double x1,double y1,double x2,double y2) {
+void SplashOutputDev::startPage(int pageNum, GfxState *state) {
   int w, h;
   double *ctm;
   SplashCoord mat[6];
@@ -2647,9 +2647,9 @@ void SplashOutputDev::setSoftMask(GfxState *state, double *bbox,
 
   softMask = new SplashBitmap(bitmap->getWidth(), bitmap->getHeight(),
 			      1, splashModeMono8, gFalse);
-  memset(softMask->getDataPtr(), 0,
-	 softMask->getRowSize() * softMask->getHeight());
+  memset(softMask->getDataPtr(), 0x00, softMask->getRowSize()*softMask->getHeight());
   p = softMask->getDataPtr() + ty * softMask->getRowSize() + tx;
+  if (tx<softMask->getWidth() && ty<softMask->getHeight())
   for (y = 0; y < tBitmap->getHeight(); ++y) {
     for (x = 0; x < tBitmap->getWidth(); ++x) {
       tBitmap->getPixel(x, y, color);
