@@ -19,7 +19,7 @@
 #ifndef WIN32
 #include <unistd.h>
 #else
-extern "C" int unlink(char *filename);
+//extern "C" int unlink(char *filename);
 #endif
 #include <string.h>
 #include <ctype.h>
@@ -2458,6 +2458,9 @@ GBool DCTStream::readProgressiveDataUnit(DCTHuffTable *dcHuffTable,
   // check for an EOB run
   if (eobRun > 0) {
     while (i <= scanInfo.lastCoeff) {
+      if(i>=64) {
+	  return gFalse;
+      }
       j = dctZigZag[i++];
       if (data[j] != 0) {
 	if ((bit = readBit()) == EOF) {
@@ -2482,6 +2485,9 @@ GBool DCTStream::readProgressiveDataUnit(DCTHuffTable *dcHuffTable,
     if (c == 0xf0) {
       k = 0;
       while (k < 16) {
+        if(i>=64) {
+	  return gFalse;
+	}
 	j = dctZigZag[i++];
 	if (data[j] == 0) {
 	  ++k;
@@ -2507,6 +2513,9 @@ GBool DCTStream::readProgressiveDataUnit(DCTHuffTable *dcHuffTable,
       }
       eobRun += 1 << j;
       while (i <= scanInfo.lastCoeff) {
+	if(i>=64) {
+	  return gFalse;
+	}
 	j = dctZigZag[i++];
 	if (data[j] != 0) {
 	  if ((bit = readBit()) == EOF) {
@@ -2529,6 +2538,9 @@ GBool DCTStream::readProgressiveDataUnit(DCTHuffTable *dcHuffTable,
       }
       k = 0;
       do {
+	if(i>=64) {
+	  return gFalse;
+	}
 	j = dctZigZag[i++];
 	while (data[j] != 0) {
 	  if ((bit = readBit()) == EOF) {
@@ -2537,6 +2549,9 @@ GBool DCTStream::readProgressiveDataUnit(DCTHuffTable *dcHuffTable,
 	  if (bit) {
 	    data[j] += 1 << scanInfo.al;
 	  }
+	  if(i>=64) {
+	    return gFalse;
+	  }
 	  j = dctZigZag[i++];
 	}
 	++k;
