@@ -448,7 +448,8 @@ static char* getTempDir()
     if(!dir) dir = getenv("temp");
     if(!dir) dir = "C:\\";
 #else
-    char* dir = "/tmp/";
+    char*dir = getenv("SWFTOOLS_TMP");
+    if(!dir) dir = "/tmp/";
 #endif
     return dir;
 }
@@ -471,7 +472,7 @@ char* mktmpname(char*ptr) {
  //   used to be mktemp. This does remove the warnings, but
  //   It's not exactly an improvement.
 #ifdef HAVE_LRAND48
-    sprintf(ptr, "%s%s%08x%08x",dir,sep,lrand48(),lrand48());
+    sprintf(ptr, "%s%s%08x%08x",dir,sep,(unsigned int)lrand48(),(unsigned int)lrand48());
 #else
 #   ifdef HAVE_RAND
 	sprintf(ptr, "%s%s%08x%08x",dir,sep,rand(),rand());
@@ -497,7 +498,7 @@ GBool openTempFile(GString **name, FILE **f, char *mode, char *ext) {
   // this has the standard race condition problem, but I haven't found
   // a better way to generate temp file names with extensions on
   // Windows
-  if ((tempDir = getenv("TEMP"))) {
+  if ((tempDir = getenv("SWFTOOLS_TMP"))) {
     s = new GString(tempDir);
     s->append('\\');
   } else {
@@ -554,7 +555,7 @@ GBool openTempFile(GString **name, FILE **f, char *mode, char *ext) {
 
   if (ext) {
 #if HAVE_MKSTEMPS
-    if ((s = getenv("TMPDIR"))) {
+    if ((s = getenv("SWFTOOLS_TMP"))) {
       *name = new GString(s);
     } else {
       *name = new GString("/tmp");
@@ -571,7 +572,7 @@ GBool openTempFile(GString **name, FILE **f, char *mode, char *ext) {
 #endif
   } else {
 #if HAVE_MKSTEMP
-    if ((s = getenv("TMPDIR"))) {
+    if ((s = getenv("SWFTOOLS_TMP"))) {
       *name = new GString(s);
     } else {
       *name = new GString("/tmp");
